#include <Wire.h>  
#include <DS1307new.h>    // Unsere Uhr
#include <DS18B20.h>
#include <LiquidCrystal_I2C.h>
#include "Tlc5940.h"
#include <EEPROMex.h>
#include <avr/pgmspace.h>


// ********************************************* //
//                 PIN MAPPING                   //
// ********************************************* //
const int TIP1 = 45;
const int TIP2 = 49;
const int TIP3 = 48;
const int TIP4 = 47;
const int TIP5 = 46;

const int DIM_C[4]={3,2,4,5};
const int TEMP = 44;
const int RELAY1 = 23;  // Licht
const int RELAY2 = 25;  // Heizung
const int RELAY3 = 27;  // Filter
const int RELAY4 = 29;  // CO2
const int RELAY5 = 31;
const int RELAY6 = 33;
const int RELAY7 = 35;
const int RELAY8 = 37;
const int LCD_BL = 13;



// ********************************************* //
//                   INITS                       //
// ********************************************* //

DS18B20_List ds18b20(TEMP);
// Create global object for menu and lcd

LiquidCrystal_I2C lcd1( 0x38 ,20,4,LCD_BL);  //  LCD Adress / Chars / Rows / PWM Pin



// ********************************************* //
//                 VARIABLES                     //
// ********************************************* //
typedef struct {
  boolean Active;    // Über diesen Kanal Dimmen
  float Sunrise;    // Zeit Sonnenaufgang
  float Sunset;    // Zeit Untergang (Dunkel)
  int Min;     // Minimum Helligkeit
  int Max;    // Max Helligkeit
  int Dim_in;    // Länge minuten eindimmen
  int Dim_out;    // Länge Minuten ausdimmen
  boolean Invert;  // Werte vertauschen
}LIGHT;
boolean moonSim=false,cloudSim=false;


#define TempOnBoard 0xC7B5
#define TempWater 0xF5
uint16_t startAddr = 0x0000;            // Start address to store in the NV-RAM
uint16_t lastAddr;                      // new address for storing in NV-RAM
uint16_t TimeIsSet = 0xaa55;            // Helper that time must not set again

float timestamp = 0;
// Hilfe für Loop ohne Delay -> mach nur was wenn es eine neue Sekunde ist
int loopTime =0;
int autoMode =1;
int o_key=-1;
int menudeep;
String menuVal;
int menulevel[4];
int DaylightLevel = 0;  // Momentanes Licht
int ManuallightLevel = 0;  // Licht bei Manueller Steuerung

#define KEYPAD01 0
#define KEYPAD02 1
#define KEYPAD03 2
#define KEYPAD04 3
#define KEYPAD11 4
#define KEYPAD12 5
#define KEYPAD13 6
#define KEYPAD14 7
#define KEYPAD21 8
#define KEYPAD22 9
#define KEYPAD23 10
#define KEYPAD24 11

// Tasten
int keypad_val[12] ={ 420, 462,497,530,
                     575,597,620,640,
                     665,682,700,715
                   };

int NUM_KEYPAD = 12;
int keypad_in;
int key=-1;
int oldkey=-1;

// ********************************************* //
//                 VARIABLES                     //
//                   EEPROM                      //
// ********************************************* //
// Dies sind nur unsere Standart einstellungen falls wir noch nie was gespeichert haben und im Menü waren.
// Sie sind für meine Zwecke angepasst, Ihr könnt Sie also so lassen oder überschreiben.
// Im Setup überprüfen wir das EEPROM und setzen die bei bedarf neu.

LIGHT light_channels[20] = { 
{ 0, get_ts(10,0,0), get_ts(23,0,0),0, 100, 30, 45,1}, 
{ 0, get_ts(10,30,0), get_ts(22,0,0),0, 100, 60, 120,1}, 
{ 0, get_ts(10,0,0), get_ts(22,0,0),0, 100, 30, 45,1}, 
{ 1, get_ts(10,0,0), get_ts(22,30,0),0, 70, 30, 45,1}, 
{ 1, get_ts(10,0,0), get_ts(22,30,0),0, 70, 30, 45,1}, 
{ 1, get_ts(10,0,0), get_ts(22,30,0),0, 70, 30, 45,1}, 
{ 1, get_ts(10,0,0), get_ts(22,30,0),0, 70, 30, 45,1}, 
{ 1, get_ts(10,0,0), get_ts(22,30,0),0, 70, 30, 45,1}, 
{ 0, get_ts(10,0,0), get_ts(20,0,0),0, 100, 30, 45,1}, 
{ 0, get_ts(10,0,0), get_ts(20,0,0),0, 100, 30, 45,1}, 
{ 0, get_ts(10,0,0), get_ts(20,0,0),0, 100, 30, 45,1}, 
{ 0, get_ts(10,0,0), get_ts(20,0,0),0, 100, 30, 45,1}, 
{ 0, get_ts(10,0,0), get_ts(20,0,0),0, 100, 30, 45,1}, 
{ 0, get_ts(10,0,0), get_ts(20,0,0),0, 100, 30, 45,1}, 
{ 0, get_ts(10,0,0), get_ts(20,0,0),0, 100, 30, 45,1}, 
{ 0, get_ts(10,0,0), get_ts(20,0,0),0, 100, 30, 45,1}, 
/*Das ganze 16 mal, dann 4 mal für 10V Dimmung*/
{ 0, get_ts(10,0,0), get_ts(23,0,0),0, 100, 30, 45,0}, 
{ 0, get_ts(10,0,0), get_ts(22,0,0),0, 100, 30, 45,0}, 
{ 0, get_ts(10,0,0), get_ts(21,0,0),0, 100, 30, 45,0}, 
{ 0, get_ts(10,0,0), get_ts(23,0,0),0, 100, 30, 120,0}
};

//Abweichung für Tageslicht basierten Verlauf
int LightOffset[24]={
  0,0,0,0,0,0,
  0,0,0,0,0,0,
  0,0,0,0,0,0,
  0,0,0,0,0,0
};

//WasserTemperatur
float Water_Temp = 25.5;

// Düngermengen
float duenger1 = 3.00;    // Spezial N 1ml/50L
float duenger2 = 1.00;    // Spezial Flowgrow 2ml/100L
float duenger3 = 2.00;    // Mikro Basis Eisen 2ml/100L
float duenger4 = 1.00;
float duenger5 = 3.00;    // Estimate Index 5ml/40l
//Dünger Zeiten
float d1Time= get_ts(9,0,0);    // 9:00
float d2Time= get_ts(10,0,0);
float d3Time= get_ts(11,0,0);    // 11:00
float d4Time= get_ts(20,0,0);    // 18:00
float d5Time= get_ts(21,0,0);    // 12:00
//Achtung, dies ist keine Standart Variable, Sie kommt von mir!
float d2Time1= get_ts(15,0,0);




// ********************************************* //
//                   Main Code                   //
// ********************************************* //


void setup() {
  Serial.begin(9600);    // Ausgabe am PC
  pinMode(2, INPUT);                    // Test of the SQW pin, D2 = INPUT
  digitalWrite(2, HIGH);                // Test of the SQW pin, D2 = Pullup on
  
  lcd1.init();                      // initialize the lcd 
  lcd1.backlight(0);    // Backlight aus

  digitalWrite((54), HIGH);  // enable the 20k internal pullup for MEGA board for analog port A0


  setupClock();
  
  Tlc.init();
  
  // Setze Pins auf Output Pins
  pinMode(TIP1, OUTPUT);
  pinMode(TIP2, OUTPUT);
  pinMode(TIP3, OUTPUT);
  pinMode(TIP4, OUTPUT);
  pinMode(TIP5, OUTPUT);
  pinMode(RELAY1, OUTPUT);
  pinMode(RELAY2, OUTPUT);
  pinMode(RELAY3, OUTPUT);
  pinMode(RELAY4, OUTPUT);
  pinMode(RELAY5, OUTPUT);
  pinMode(RELAY6, OUTPUT);
  pinMode(RELAY7, OUTPUT);
  pinMode(RELAY8, OUTPUT);
 
 
  digitalWrite(RELAY1, HIGH);
  digitalWrite(RELAY2, HIGH);
  digitalWrite(RELAY3, HIGH);
  digitalWrite(RELAY4, HIGH);
  digitalWrite(RELAY5, HIGH);
  digitalWrite(RELAY6, HIGH);
  digitalWrite(RELAY7, HIGH);
  digitalWrite(RELAY8, HIGH);

////////////////////////////////////////////////////
//                                                //
//             ACHTUNG!!!!!!!!!!!!!!!             //
//                                                //
////////////////////////////////////////////////////
// Diese Zeile Läd und Speichert Daten ins EEPROM.
// Das ist aber nur 100.000 mal möglich!!! Klammert
// diese Zeile also nur aus wenn Ihr sicher seit das 
// Ihr vom Testen in Voll betrieb wollt und über das
// Keypad Variablen ändern wollt.

//  LoadEEPROM(); 
  
}

void loop() {
	RTC.getTime();
   main_Screen();
	ds18b20.update(); // Temperatur update
        Duengung(RTC.daystamp);
        
        if(autoMode==1){ 
          int abl=DaylightLevel*2.55;
          if(abl<5){abl=5;}
          lcd1.backlight(abl);            
    
          // Nun schauen wir wie hell es sein soll
          setLight(0);  // Wir übergeben 0 damit automatisch nen Value errechnet wird
        }
            
	if(loopTime !=int(RTC.second)){
	    loopTime =int(RTC.second);
		Serial.println (printTime());
		Serial.println (printDate());

            // Temperaturen auslesen
                // Falls man die ID der Sensoren nicht kennt hilft der Code hier:
            /*
                for (int i=0;i<ds18b20.count;i++)
                  {
                  Serial.print("Sensor ");
                  Serial.print(i,DEC);
                  Serial.print(", id=0x");
                  Serial.print(ds18b20.get_id(i),HEX);
                  Serial.print(", T=");
                  print_temperature(ds18b20.get_temp(i));
                  Serial.println("C");
                  }
                  // kann gelöscht werden sobalt die Sensoren eingestellt sind.
    */
		float T_inside=ds18b20.get_temp_by_id(TempOnBoard);
		float T_water=ds18b20.get_temp_by_id(TempWater);

                Luefter(T_inside);
                
                // Heizung Steuern
                if(Water_Temp-0.5>T_water && digitalRead(RELAY2)==HIGH){
                  digitalWrite(RELAY2,LOW);
                }else if(Water_Temp+0.5<T_water && digitalRead(RELAY2)==LOW){
                  digitalWrite(RELAY2,HIGH);
                }
        }  
 key=keypad();   
 if(menudeep!=0){
   if(key!=o_key){
   o_key=key;
     showMenu();
   }
 }else{
 }
}
